<?php

/**
 * Handle AJAX actions
 */
class Emlg_Ajax {

	/**
	 * Construct an instance
	 */
	public function __construct() {
		add_action( 'wp_ajax_emlg_check_email_cap', [ $this, 'check_email_capability' ] );
		add_action( 'wp_ajax_emlg_preview_login_email', [ $this, 'preview_login_email' ] );
		add_action( 'wp_ajax_emlg_forms', [ $this, 'forms' ] );
	}

	/**
	 * Return the content of the template editor to be viewed in an iframe
	 *
	 * @return void
	 */
	public function preview_login_email() {
		$vars    = self::ajax_checks( 'emlg' );
		$content = isset( $vars['content'] ) ? $vars['content'] : '';
		echo wp_kses( str_ireplace( [ '%SITE_TITLE%', '%CODE%', '%USER%' ], [ get_bloginfo(), 'XXXXXX', wp_get_current_user()->data->user_login ], $content ), $this->email_allowed_tags() );
		die;
	}

	/**
	 * Process all form submitted by AJAX
	 *
	 * @return void
	 */
	public function forms() {
		$vars = self::ajax_checks( 'emlg' );
		parse_str( $vars['args'], $args );

		if ( method_exists( $this, $args['form_name'] ) && is_callable( [ $this, $args['form_name'] ] ) ) {
			$results = call_user_func( [ $this, $args['form_name'] ], $args );
			wp_send_json( [ 'data' => $results, 'success' => true ], 200 );
		}

		wp_send_json_error( 'Not found', 400 );
	}

	/**
	 * Email capability check form (code submission).
	 *
	 * @param array $args from inputs.
	 *
	 * @return array the new plugin option.
	 */
	private function submit_code( $args ) {
		$code = new Emlg_Codes( $args['code'], 'mail_cap' );

		if ( $code->check() === false ) {
			return [
				'template'       => 'emlg-mail-check',
				'template_error' => __( 'The code is incorrect or has expired.', 'emlg-tfa' ),
			];
		}

		$options = Emlg_Login::update_option( 'mail_cap_status', 'yes' );

		return [
			'template'    => 'emlg-settings',
			'new_options' => $options,
		];
	}

	/**
	 * Plugin settings form
	 *
	 * @param array $args from inputs.
	 *
	 * @return array the new plugin option.
	 */
	private function save_settings( $args ) {
		$options = Emlg_Login::get_option();

		foreach ( $args as $key => $value ) {
			if ( is_array( $value ) && isset( $options[ $key ] ) && is_array( $options[ $key ] ) ) {
				foreach ( $value as $index => $content ) {
					if ( isset( $options[ $key ][ $index ] ) ) {
						if ( $key === 'email_details' ) {
							$options[ $key ][ $index ] = ( $index === 'sender_email' ) ? sanitize_email( $content ) : sanitize_text_field( $content );
						}

						if ( $key === 'templates' && $index === 'login_mail' && stripos( $content, '%CODE%' ) !== false ) {
							// Do not modify the template if there is no placeholder for the code.
							$options[ $key ][ $index ] = wp_kses( $content, $this->email_allowed_tags() );
						}
					}
				}
			}

			if ( is_string( $value ) && isset( $options[ $key ] ) && is_string( $options[ $key ] ) ) {
				$options[ $key ] = sanitize_text_field( $value );
			}
		}

		Emlg_Login::update_option( $options );

		return [
			'template'    => 'emlg-settings',
			'new_options' => $options,
		];
	}

	/**
	 * Get allowed HTML tags in email content
	 *
	 * @return array
	 */
	public function email_allowed_tags() {
		$tags = [
			'a'     => [],
			'div'   => [],
			'span'  => [],
			'table' => [],
			'thead' => [],
			'tfoot' => [],
			'th'    => [],
			'tr'    => [],
			'td'    => [],
			'p'     => [],
			'i'     => [],
			'style' => [],
			'em'    => [],
			'br'    => [],
			'hr'    => [],
			'img'   => [],
		];

		$allowed_tags = array_map( function( $attributes ) {
			$attributes['style'] = [];
			$attributes['id']    = [];
			$attributes['class'] = [];
			$attributes['title'] = [];

			return $attributes;
		}, $tags );

		$allowed_tags['img']['src']  = [];
		$allowed_tags['img']['alt']  = [];
		$allowed_tags['a']['href']   = [];
		$allowed_tags['a']['target'] = [];
		$allowed_tags['a']['rel']    = [];

		return $allowed_tags;
	}

	/**
	 * Attempt to send a test email
	 *
	 * @return void
	 */
	public function check_email_capability() {
		self::ajax_checks( 'emlg' );
		$options = Emlg_Login::get_option();
		$code    = Emlg_Codes::random_code();

		while ( Emlg_Codes::code_exists( $code ) ) {
			$code = Emlg_Codes::random_code();
		}

		$content = '<div style=\'font-family:-apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif;color:#fff;background-color:#2271b1;padding:10px;\'>' . "\n" .
				   "\t" . '<p style="font-size:1.6em;">%SITE_TITLE%</p>' . "\n" .
				   "\t" . '<hr/>' . "\n" .
				   "\t" . '<p>Checking for email capability of WordPress at %SITE_URL%</p>' . "\n" .
				   "\t" . '<p style="font-size: 1.1em;">Enter the following code on the EMLG TFA dashboard page:</p>' . "\n" .
				   "\t" . '<p style="font-size: 1.6em;font-weight: 700">%CODE%</p>' . "\n" .
				   '</div>';

		$content             = str_replace( [ '%SITE_TITLE%', '%CODE%', '%SITE_URL%' ], [ get_bloginfo(), $code, site_url() ], $content );
		$email               = new Emlg_Email( $content );
		$email->sender_email = $options['email_details']['sender_email'];
		$email->sender_name  = $options['email_details']['sender_name'];
		$email->subject      = $options['email_details']['subject'];
		$user                = wp_get_current_user();
		$email->to           = $user->user_email;
		$success             = $email->send();

		if ( $success === true ) {
			$code_object = new Emlg_Codes( $code, 'mail_cap' );
			$code_object->store( $user->user_email );
			$options['mail_cap_status'] = $user->user_email;
			Emlg_Login::update_option( $options );
			wp_send_json_success( [ 'email' => $user->user_email, 'new_options' => $options ], 200 );
		}

		$error_message = ! empty( $success ) ? $success : __( 'WordPress could not send the email', 'emlg-tfa' );

		wp_send_json_error(
			[
				'error'    => $error_message,
				'template' => 'emlg-mail-check',
			],
			400
		);
	}

	/**
	 * Perform AJAX security checks - nonce and user capability
	 *
	 * @param string $action nonce action
	 *
	 * @return array
	 */
	public static function ajax_checks( $action ) {
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_send_json_error( 'Forbidden', 403 );
		}

		$nonce = isset( $_POST['nonce'] ) ? sanitize_key( $_POST['nonce'] ) : '';

		if ( ! wp_verify_nonce( $nonce, $action ) ) {
			wp_send_json_error( 'Unauthorized', 400 );
		}

		return wp_unslash( $_POST );
	}
}
