<?php

/**
 * Handle all operations on confirmation codes
 */
class Emlg_Codes {
	/**
	 * Name of the option where codes are stored
	 *
	 * @const
	 */
	const OPTION = 'emlg_codes';

	/**
	 * The code string
	 *
	 * @var string
	 */
	private $code;

	/**
	 * What the code is for (login or email capability check)
	 *
	 * @var string
	 */
	private $type;

	/**
	 * Construct an instance of the class
	 *
	 * @param string $code the code.
	 * @param string $type the code type.
	 */
	public function __construct( $code, $type ) {
		$this->code = $code;
		$this->type = $type === 'login' ? 'login' : 'mail_cap';
	}

	/**
	 * Stores the code in DB with all the relevant data like the user and the validity period
	 *
	 * @param mixed $data the entire data to record in DB.
	 *
	 * @return void
	 */
	public function store( $data ) {
		$codes                = self::get_codes();
		$codes[ $this->code ] = [
			'expires' => time() + 10 * MINUTE_IN_SECONDS,
			'type'    => $this->type,
			'data'    => $data,
		];
		update_option( self::OPTION, $codes );
	}

	/**
	 * Check if the code passed in the constructor is valid.
	 * Automatically delete the entry if the code has expired.
	 *
	 * @return false|string
	 */
	public function check() {
		$codes = self::get_codes();

		if ( ! isset( $codes[ $this->code ] ) ) {
			return false;
		}

		if ( $codes[ $this->code ]['type'] != $this->type ) {
			return false;
		}

		$data = $codes[ $this->code ]['data'];
		unset( $codes[ $this->code ] );
		update_option( self::OPTION, $codes );

		return $data;
	}

	/**
	 * Get all codes entries in DB
	 *
	 * @return array
	 */
	private static function get_codes() {
		$raw_codes = get_option( self::OPTION, [] );
		$codes     = [];
		$now       = time();

		foreach ( $raw_codes as $code => $value ) {
			if ( isset( $value['expires'] ) && (int) $value['expires'] > $now ) {
				// Keep only codes that haven't expired yet.
				$codes[ $code ] = $value;
			}
		}

		if ( count( $raw_codes ) !== count( $codes ) ) {
			update_option( self::OPTION, $codes );
		}

		return $codes;
	}

	/**
	 * Check if a given code matches something in DB
	 *
	 * @param string $code the code to check.
	 *
	 * @return bool
	 */
	public static function code_exists( $code ) {
		$codes = self::get_codes();

		return isset( $codes[ $code ] );
	}

	/**
	 * Generate a random string
	 *
	 * @param int $length conde length.
	 *
	 * @return string
	 */
	public static function random_code( $length = 6 ) {
		return strtolower( wp_generate_password( $length, false, false ) );
	}

}
