<?php

/**
 * Handle the login process
 */
class Emlg_Login {

	/**
	 * Login error message
	 *
	 * @var string
	 */
	private static $error_message;

	/**
	 * The unique instance
	 *
	 * @var Emlg_Login
	 */
	private static $instance;

	/**
	 * Plugin option name
	 *
	 * @const
	 */
	const OPTION_NAME = 'emlg_options';

	/**
	 * Build the singleton
	 */
	private function __construct() {
		add_action( 'wp_authenticate', [ $this, 'authenticate' ], 100, 2 );
		add_filter( 'template_include', [ $this, 'template_include' ], 100 );
	}

	/**
	 * Get the last login error message
	 *
	 * @return string
	 */
	public static function get_error_message() {
		return isset( self::$error_message ) ? self::$error_message : '';
	}

	/**
	 * Modify the template to include when a login code was sent
	 *
	 * @param string $template path to the current template.
	 *
	 * @return string|void
	 */
	public function template_include( $template ) {
		if ( isset( $_POST['emlg_code'] ) ) {
			// A login code has been submitted.
			$code        = sanitize_text_field( $_POST['emlg_code'] );
			$code_object = new Emlg_Codes( $code, 'login' );

			// Check if the code correspond to a set of credentials and has not expired yet.
			$stored_data = $code_object->check();

			if ( $stored_data === false ) {
				self::$error_message = __( 'The code is invalid or has expired', 'emlg-tfa' );

				return EMLG_PATH . 'views/submit-code.php';
			}

			$stored_data = json_decode( base64_decode( $stored_data ), true );

			if ( isset( $stored_data['credentials'] ) ) {
				// Remove the hooked function first. "wp_authenticate" is fired during wp_signon().
				remove_action( 'wp_authenticate', [ $this, 'authenticate' ], 100, 2 );

				// Log the user in using the stored credentials.
				wp_signon( $stored_data['credentials'] );

				$redirect_url = ! empty( $stored_data['redirect_to'] ) ? $stored_data['redirect_to'] : admin_url();
				wp_redirect( $redirect_url );
				exit;
			}

			self::$error_message = __( 'The code is invalid or has expired', 'emlg-tfa' );

			return EMLG_PATH . 'views/submit-code.php';
		} elseif ( isset( $_GET['emlg_code'] ) ) {
			$template = EMLG_PATH . 'views/submit-code.php';
		}

		return $template;
	}

	/**
	 * Get plugin's default options
	 *
	 * @return array
	 */
	private static function get_default_options() {
		return [
			'mail_cap_status' => '',
			'templates'       => [
				'login_mail' => '<div style=\'font-family:-apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif;color:#fff;background-color:#2271b1;padding:10px;\'>' . "\n" .
								"\t" . '<p style="font-size:1.6em;">%SITE_TITLE%</p>' . "\n" .
								"\t" . '<hr/>' . "\n" .
								"\t" . '<p>A login attempt to %SITE_URL% with your account "%USER%" has been performed.</p>' . "\n" .
								"\t" . '<p style="font-size: 1.1em;">This is your login code:</p>' . "\n" .
								"\t" . '<p style="font-size: 1.6em;font-weight: 700">%CODE%</p>' . "\n" .
								"\t" . '<hr/>' . "\n" .
								"\t" . '<p>If you did not perform this action, your password is likely compromised.</p>' . "\n" .
								'</div>',
			],
			'email_details'   => [
				'sender_name'  => 'Email login',
				'sender_email' => 'noreply@example.com',
				'subject'      => 'Login verification',
			],
			'enabled'         => '',
		];
	}

	/**
	 * Return plugin option
	 *
	 * @return array
	 */
	public static function get_option() {
		$options = get_option( self::OPTION_NAME, [] );

		return wp_parse_args( $options, self::get_default_options() );
	}

	/**
	 * Update the entire option or a single field of it
	 *
	 * @param string|array $field_or_options the entire array of new option or the name of the field to update.
	 * @param string|array $value            new value of the field to update.
	 *
	 * @return array
	 */
	public static function update_option( $field_or_options, $value = null ) {
		$options = Emlg_Login::get_option();

		if ( is_string( $field_or_options ) && $value !== null && isset( self::get_default_options()[ $field_or_options ] ) ) {
			// Updating a single field.
			$options[ $field_or_options ] = $value;
			update_option( self::OPTION_NAME, $options );

			return $options;
		}

		if ( is_array( $field_or_options ) ) {
			// Updating the entire option.
			$new_options = $field_or_options + $options;
			update_option( self::OPTION_NAME, $new_options );

			return $new_options;
		}

		return $options;
	}

	/**
	 * Hooked into "wp_authenticate"
	 *
	 * If login and password are correct, store credentials, send email then reload "wp-login.php" and display the "Enter code" form.
	 * If logging in fails, let "wp-login.php" do its usual job.
	 *
	 * @param string $login
	 * @param string $pwd
	 *
	 * @return void
	 */
	public function authenticate( $login, $pwd ) {
		$current_user = wp_get_current_user();

		if ( $current_user->ID !== 0 || ( isset( $_REQUEST['action'] ) && $_REQUEST['action'] !== 'login' ) || isset( $_REQUEST['interim-login'] ) ) {
			// Do not mess with logged-in user, other action (like reset password) and expired session modal frame ("interim-login").
			return;
		}

		$options = self::get_option();

		if ( $options['enabled'] !== 'enabled' ) {
			return;
		}

		if ( ! empty ( $login ) && ! empty( $pwd ) ) {
			$credentials = [
				'user_login'    => $login,
				'user_password' => $pwd,
			];

			if ( ! empty( $_POST['rememberme'] ) ) {
				$credentials['remember'] = sanitize_key( $_POST['rememberme'] );
			}

			$user = wp_authenticate( $credentials['user_login'], $credentials['user_password'] );

			$redirect   = isset( $_POST['redirect_to'] ) ? sanitize_url( $_POST['redirect_to'] ) : '';
			$rememberme = isset( $_POST['rememberme'] ) ? sanitize_key( $_POST['rememberme'] ) : '';

			if ( ! is_wp_error( $user ) ) {
				// Username and password are correct, do not set authentication cookie, send the email and display the login code form.
				$this->correct_credentials( $user, $credentials, $redirect, $rememberme );
			}
		}
	}

	/**
	 * Username and password matched a user. Create a login code
	 *
	 * @param WP_User $user        the user that logged in.
	 * @param array   $credentials username, password, remember me.
	 * @param string  $redirect    page from where the user comes from before logging in.
	 * @param string  $rememberme  stay connected (remember me field).
	 *
	 * @return void
	 */
	private function correct_credentials( $user, $credentials, $redirect, $rememberme ) {
		$email = Emlg_Email::email_hint( $user->user_email );

		// Create a code and attempt to send it via email.
		$code_generated = $this->generate_code( $user, $credentials, $redirect, $rememberme );

		if ( $code_generated ) {
			// Email sent, redirect.
			wp_redirect( site_url( '?emlg_code=' . base64_encode( $email ) ) );
			exit;
		}
	}

	/**
	 * Create a code and try to send it via email.
	 *
	 * @param WP_User $user        the user that logged in.
	 * @param array   $credentials the user credentials.
	 * @param string  $redirect    redirect url.
	 * @param string  $rememberme  stay logged in.
	 *
	 * @return bool
	 */
	private function generate_code( $user, $credentials, $redirect, $rememberme ) {
		$code = Emlg_Codes::random_code();

		// Replace all placeholder in the email message.
		$email = new Emlg_Email( str_ireplace( [ '%CODE%', '%USER%', '%SITE_TITLE%', '%SITE_URL%' ], [ $code, $credentials['user_login'], get_bloginfo(), site_url() ], self::get_option()['templates']['login_mail'] ) );

		$email->to           = $user->user_email;
		$details             = $this->get_code_email_details();
		$email->sender_name  = $details['sender_name'];
		$email->sender_email = $details['sender_email'];
		$email->subject      = $details['subject'];

		if ( ! empty( $rememberme ) ) {
			$credentials['rememberme'] = $rememberme;
		}

		$login_data = [
			'credentials' => $credentials,
			'redirect_to' => $redirect,
		];

		$success = $email->send();

		if ( $success === true ) {
			// Email sent.
			$code_object = new Emlg_Codes( $code, 'login' );
			$code_object->store( base64_encode( json_encode( $login_data ) ) );

			return true;
		}

		// Email failure, disable Email 2FA option. (otherwise admins might be locked out of the dashboard).
		self::update_option( 'mail_cap_status', 'no' );

		return false;
	}

	/**
	 * Get login email sender name, address and email subject
	 *
	 * @return array
	 */
	public function get_code_email_details() {
		return self::get_option()['email_details'];
	}

	/**
	 * Return the unique instance
	 *
	 * @return Emlg_Login
	 */
	public static function get_instance() {
		if ( self::$instance === null ) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}

Emlg_Login::get_instance();
