( function ( $, wp ) {
	'use strict';

	let pickTemplateFromTarget = false, overlay;

	/**
	 * HTML editor (CodeMirror).
	 */
	const editor = {
		target: '',
		frame:  null,
		editor: null,

		selectAction: function ( action ) {
			if ( this.editor === null ) {
				// First time opening the HTML editor.
				this.frame  = $( '#emlg-editor' );
				this.editor = wp.codeEditor.initialize( this.frame.find( 'textarea' ), window.emlg_codemirror_settings );
				const self  = this;
				this.frame.find( '.head [data-action="apply"]' ).on( 'click', function () {
					self.target.val( self.editor.codemirror.doc.getValue() );
					self.hide();
				} );
				this.frame.find( '.head [data-action]' ).on( 'click', function () {
					self.selectAction( $( this ).attr( 'data-action' ) );
				} );
			}

			if ( action === 'edit' ) {
				if ( $( '#emlg-editor .body .content[data-action="preview"]' ).is( ':visible' ) ) {
					$( '#emlg-editor .body .content' ).hide();
					$( '#emlg-editor .body .content[data-action="edit"]' ).show();
				}

				if ( this.editor.codemirror.doc.getValue() === '' || pickTemplateFromTarget === true ) {
					// Editor content is empty or after saving (and sanitizing), pick the content from the hidden text input.
					pickTemplateFromTarget = false;
					this.editor.codemirror.doc.setValue( this.target.val() );
				}

				$( '#emlg-editor .head [data-action="edit"]' ).addClass( 'disabled' );
				$( '#emlg-editor .head [data-action="preview"]' ).removeClass( 'disabled' );
			}

			if ( action === 'preview' ) {
				$( '#emlg-editor .head [data-action="edit"]' ).removeClass( 'disabled' );
				$( '#emlg-editor .head [data-action="preview"]' ).addClass( 'disabled' );
				$( '#emlg-editor .body .content' ).hide();
				$( '#emlg-editor .body .content[data-action="preview"]' ).show();

				wp.ajax.send( 'emlg_preview_login_email', {
					data:     {
						nonce:   $( '#emlg' ).attr( 'data-nonce' ),
						content: this.editor.codemirror.doc.getValue()
					},
					complete: function ( response ) {
						if ( typeof response.responseText !== 'undefined' ) {
							$( '#emlg-editor .body [data-action="preview"] iframe' ).contents().find( 'body' ).html( response.responseText );
							resize();
						}
					}
				} );
			}
		},

		show: function ( action, target ) {
			if ( this.frame === null ) {
				this.frame = $( '#emlg-editor' );
			}
			this.frame.show();
			$( '#emlg-editor .body .content' ).hide();
			$( '#emlg-editor .body .content[data-action="edit"]' ).show();
			editor.target = $( '#' + target );
			this.selectAction( action );
		},

		hide: function () {
			this.frame.hide();
		}
	};

	// Click on close icon or the overlay background.
	$( document ).on( 'click', '.emlg-close-editor', function () {
		editor.hide();
	} );

	// Prevent event bubbling (up to the overlay background which will close the editor).
	$( document ).on( 'click', '#emlg-editor .inner-frame', function ( ev ) {
		ev.stopPropagation();
	} );

	// Open the editor.
	$( document ).on( 'click', '.emlg-open-editor', function () {
		editor.show( 'edit', $( this ).attr( 'data-target' ) );
	} );

	// Save options (AJAX).
	$( document ).on( 'submit', '#emlg', function ( e ) {
		e.preventDefault();
		const serialized = $( this ).serialize();
		if ( serialized.indexOf( 'form_name=' ) !== - 1 ) {
			overlay.show();
			wp.ajax.send( 'emlg_forms', {
				data:    {
					nonce: $( '#emlg' ).attr( 'data-nonce' ),
					args:  serialized
				},
				success: function ( data ) {
					overlay.hide();
					updateOptions( data );
					if ( typeof data.template !== 'undefined' ) {
						let templateData = {options: window.emlg_options};
						if ( typeof data.template_error !== 'undefined' ) {
							templateData.error = data.template_error;
						}
						updateMarkup( data.template, templateData );
						pickTemplateFromTarget = true;
					}
				}
			} );
		}
	} );

	$( document ).on( 'click', '#emlg-check-email-capability', checkEmailCap );

	// Resize the modal frame
	function resize() {
		const body = $( '#emlg-editor .body' );
		if ( body.is( ':hidden' ) ) {
			return;
		}
		body.height( $( '#emlg-editor .inner-frame' ).outerHeight() - $( '#emlg-editor .head' ).outerHeight() );
		const iframe = body.find( 'iframe' );
		if ( iframe.is( ':hidden' ) ) {
			return;
		}
		iframe.height( body.height() * .95 );
	}

	// Send the test email.
	function checkEmailCap() {
		overlay.show();
		wp.ajax.send( 'emlg_check_email_cap', {
				data:    {
					nonce: $( '#emlg' ).attr( 'data-nonce' )
				},
				success: function ( data ) {
					overlay.hide();
					updateOptions( data );
					updateMarkup( 'emlg-mail-check', {options: window.emlg_options} );
				},
				error:   function ( response ) {
					overlay.hide();
					if ( typeof response.responseJSON !== 'undefined'
						&& typeof response.responseJSON.data !== 'undefined'
						&& typeof response.responseJSON.data.template !== 'undefined'
					) {
						console.log( response.responseJSON.data.error );
						let templateData = {options: window.emlg_options};
						if ( typeof response.responseJSON.data.error !== 'undefined' ) {
							templateData.error = response.responseJSON.data.error;
						}
						updateMarkup( response.responseJSON.data.template, templateData );
					}
				}
			}
		);
	}

	/**
	 * Update global option variable
	 *
	 * @param data new options value (saving form via AJAX).
	 */
	function updateOptions( data ) {
		if ( typeof data.new_options !== 'undefined' ) {
			window.emlg_options = data.new_options;
		}
	}

	/**
	 * Update the setting page markup
	 *
	 * @param template template ID to display.
	 * @param data template variables.
	 */
	function updateMarkup( template, data ) {
		$( '#emlg' ).html( ( wp.template( template ) )( data ) );
	}

	/**
	 * DOM ready.
	 */
	$( function () {
		overlay = $( '#emlg-overlay' );
		$( '#wpwrap' ).append( overlay.add( $( '#emlg-editor' ) ) );
		if ( window.emlg_options.mail_cap_status !== 'yes' ) {
			updateMarkup( 'emlg-mail-check', {options: window.emlg_options} );
		} else {
			updateMarkup( 'emlg-settings', {options: window.emlg_options} );
		}
	} );

	$( window ).resize( resize );

} )( window.jQuery, window.wp );
